@@ -421,26 +421,12 @@ static gboolean
 terminal_window_dropdown_can_grab (gpointer data)
 {
   GdkWindow    *window = gtk_widget_get_window (GTK_WIDGET (data));
-  GdkGrabStatus status;
-
-#if GTK_CHECK_VERSION (3, 20, 0)
-  GdkSeat *seat = gdk_display_get_default_seat (gtk_widget_get_display (GTK_WIDGET (data)));
-  if (!gdk_window_is_visible (window))
-    return FALSE;
-  status = gdk_seat_grab (seat, window, GDK_SEAT_CAPABILITY_KEYBOARD, FALSE, NULL, NULL, NULL, NULL);
-#else
-  status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
-#endif
+  GdkGrabStatus status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
 
   if (status == GDK_GRAB_SUCCESS)
     {
       /* drop the grab */
-#if GTK_CHECK_VERSION (3, 20, 0)
-      gdk_seat_ungrab (seat);
-#else
       gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-#endif
-
       return FALSE;
     }
 
@@ -511,7 +497,7 @@ terminal_window_dropdown_status_icon_press_event (GtkStatusIcon          *status
   if (event->button == 3)
     return FALSE;
 
-  if (gtk_widget_get_visible (GTK_WIDGET (dropdown)))
+  if (gdk_window_is_visible (gtk_widget_get_window (GTK_WIDGET (dropdown))))
     terminal_window_dropdown_hide (dropdown);
   else
     terminal_window_dropdown_show (dropdown, event->time);
@@ -547,10 +533,14 @@ terminal_window_dropdown_status_icon_popup_menu (GtkStatusIcon          *status_
                          gtk_action_create_menu_item (action));
 
   gtk_widget_show_all (menu);
+#if GTK_CHECK_VERSION (3, 22, 0)
+  gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
+#else
   gtk_menu_popup (GTK_MENU (menu),
                   NULL, NULL,
                   NULL, NULL,
                   button, timestamp);
+#endif
 }
 
 
@@ -565,7 +555,14 @@ terminal_window_dropdown_animate_down (gpointer data)
   gint                    step_size, vbox_h;
 
   /* get window size */
+#if GTK_CHECK_VERSION (3, 22, 0)
+  GdkDisplay *display = gdk_screen_get_display (dropdown->screen);
+  GdkMonitor *monitor =
+      gdk_display_get_monitor_at_window (display, gtk_widget_get_window (GTK_WIDGET (data)));
+  gdk_monitor_get_geometry (monitor, &rect);
+#else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &rect);
+#endif
   if (!gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
     {
       /* calculate width/height if not fullscreen */
@@ -612,7 +609,14 @@ terminal_window_dropdown_animate_up (gpointer data)
   gint                    step_size, vbox_h, min_size;
 
   /* get window size */
+#if GTK_CHECK_VERSION (3, 22, 0)
+  GdkDisplay *display = gdk_screen_get_display (dropdown->screen);
+  GdkMonitor *monitor =
+      gdk_display_get_monitor_at_window (display, gtk_widget_get_window (GTK_WIDGET (data)));
+  gdk_monitor_get_geometry (monitor, &rect);
+#else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &rect);
+#endif
   if (!gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
     {
       /* calculate width/height if not fullscreen */
@@ -706,8 +710,12 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   gboolean           visible;
   gint               vbox_h;
   TerminalDirection  old_animation_dir = ANIMATION_DIR_NONE;
+#if GTK_CHECK_VERSION (3, 22, 0)
+  GdkDisplay        *display;
+  GdkMonitor        *monitor;
+#endif
 
-  visible = gtk_widget_get_visible (GTK_WIDGET (dropdown));
+  visible = gdk_window_is_visible (gtk_widget_get_window (GTK_WIDGET (dropdown)));
 
   if (dropdown->animation_timeout_id != 0)
     {
@@ -726,7 +734,14 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
     dropdown->screen = xfce_gdk_screen_get_active (&dropdown->monitor_num);
 
   /* get the active monitor size */
+#if GTK_CHECK_VERSION (3, 22, 0)
+  display = gdk_screen_get_display (dropdown->screen);
+  monitor = gdk_display_get_monitor_at_window (display,
+                                               gtk_widget_get_window (GTK_WIDGET (dropdown)));
+  gdk_monitor_get_geometry (monitor, &monitor_geo);
+#else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &monitor_geo);
+#endif
 
   /* move window to correct screen */
   gtk_window_set_screen (GTK_WINDOW (dropdown), dropdown->screen);
@@ -817,7 +832,7 @@ terminal_window_dropdown_toggle_real (TerminalWindowDropdown *dropdown,
   gboolean        toggle_focus;
 
   if (!force_show
-      && gtk_widget_get_visible (GTK_WIDGET (dropdown))
+      && gdk_window_is_visible (gtk_widget_get_window (GTK_WIDGET (dropdown)))
       && dropdown->animation_dir != ANIMATION_DIR_UP)
     {
       g_object_get (G_OBJECT (window->preferences), "dropdown-toggle-focus", &toggle_focus, NULL);
@@ -929,10 +944,6 @@ terminal_window_dropdown_new (const gchar        *role,
   g_signal_connect_swapped (action, "activate",
       G_CALLBACK (terminal_window_dropdown_update_geometry), window);
 
-
-  g_signal_connect_swapped (window->action_fullscreen, "activate",
-      G_CALLBACK (terminal_window_dropdown_update_geometry), window);
-
   return GTK_WIDGET (window);
 }
 
@@ -968,10 +979,21 @@ terminal_window_dropdown_get_size (TerminalWindowDropdown *dropdown,
   gint            xpad, ypad;
   glong           char_width, char_height;
   GtkRequisition  req;
+#if GTK_CHECK_VERSION (3, 22, 0)
+  GdkDisplay     *display;
+  GdkMonitor     *monitor;
+#endif
 
   /* get the active monitor size */
   gdkscreen = xfce_gdk_screen_get_active (&monitor_num);
+#if GTK_CHECK_VERSION (3, 22, 0)
+  display = gdk_screen_get_display (gdkscreen);
+  monitor = gdk_display_get_monitor_at_window (display,
+                                               gtk_widget_get_window (GTK_WIDGET (dropdown)));
+  gdk_monitor_get_geometry (monitor, &monitor_geo);
+#else
   gdk_screen_get_monitor_geometry (gdkscreen, monitor_num, &monitor_geo);
+#endif
 
   /* get terminal size */
   terminal_screen_get_geometry (screen, &char_width, &char_height, &xpad, &ypad);
